#!/bin/sh
#
# Oreon is developped with Apache Licence 2.0 : 
# http://www.apache.org/licenses/LICENSE-2.0.txt 
# Developped by : Jean-Baptiste Gouret - Julien Mathis - Romain Le Merlus 
#                 Christophe Coraboeuf - Mathieu Chateau 

# The Software is provided to you AS IS and WITH ALL FAULTS. 
# OREON makes no representation and gives no warranty whatsoever, 
# whether express or implied, and without limitation, with regard to the quality, 
# safety, contents, performance, merchantability, non-infringement or suitability for 
# any particular or intended purpose of the Software found on the OREON web site. 
# In no event will OREON be liable for any direct, indirect, punitive, special, 
# incidental or consequential damages however they may arise and even if OREON has 
# been previously advised of the possibility of such damages.

echo 
echo "##################################################"
echo "#         OREON Project (www.oreon.org)          #"
echo "#             Thanks for using OREON             #"
echo "#                                                #"
echo "#                     v 1.2.2-RC3                #"
echo "#                                                #"
echo "#                 infos@oreon.org                #"
echo "#                                                #"
echo "#  Make sure you have installed and configured   #"
echo "#                  sudo - nmap                   #"
echo "#                  php - apache                  #"
echo "#                                                #"
echo "#                                                #"
echo "##################################################"
echo "#                 The Team OREON                 #"
echo "##################################################"
echo ""
echo ""
echo "      Make sure you have root permissions !"
echo ""
echo ""
echo " WARNING : Setup will delete all previous informations in your OREON DATABASE. "
echo ""

echo "Are you sure to continue?"
echo -n "[y/n], default to [n]:"
read temp
if [ -z $temp ];then
    temp=n
fi

if [ $temp = "n" ];then
    echo "Good boy..."
    exit
fi

test_answer()
{
    #$1 variable to fill
    #$2 text typed by user
    if [ ! -z $2 ];then
        if [ $2 != "" ];then
	    eval $1=$2
        fi
    fi
}

##
## CONFIGURATION
## 

echo ""
INSTALL_DIR_NAGIOS="/usr/local/nagios"
echo "Where is installed Nagios ?"
echo -n "default to [$INSTALL_DIR_NAGIOS]:"
read temp
test_answer INSTALL_DIR_NAGIOS $temp

echo ""

#setup directory for oreon
INSTALL_DIR_OREON="/usr/local/oreon"
echo "Where do I install Oreon ?"  
echo -n "default to [$INSTALL_DIR_OREON]:"
read temp
test_answer INSTALL_DIR_OREON $temp

echo ""

#Configuration file for sudo
SUDO_FILE="/etc/sudoers"
echo "Where is sudo ?"  
echo -n "default to [$SUDO_FILE]:"
read temp
test_answer SUDO_FILE $temp


##
## VARIABLES
##
## Make sure you know what you do if you modify it !!

SRC_OREON="oreon_src"

PLUGINS_DIR="plugins"

PWD=`pwd`

LOG_FILE="$PWD/log/install_oreon.log"

date > $LOG_FILE

TRUETYPE="/usr/X11R6/lib/X11/fonts/truetype"


##
## INSTALL
##

if test -d $INSTALL_DIR_NAGIOS/libexec ; then
    echo "Nagios libexec directory ... OK"
else
    mkdir -p $INSTALL_DIR_NAGIOS/libexec > /dev/null
    echo "Nagios libexec directory ... OK"
fi

if test -d $TRUETYPE ; then
    cp truetype/verdanab.ttf $TRUETYPE/verdanab.ttf > /dev/null
    echo "TrueType verdana ... OK"
else
    mkdir -p $TRUETYPE > /dev/null
    cp truetype/verdanab.ttf $TRUETYPE/verdanab.ttf > /dev/null
    echo "TrueType verdana ... OK"
fi


# check for httpd directory

if test -d /etc/apache/conf ; then
    DIR_APACHE="/etc/apache/conf"
    DIR_APACHE_CONF="/etc/apache/conf.d"
    HTPASSWD="htpasswd"
else if test -d /usr/local/apache2/conf ; then
    DIR_APACHE="/usr/local/apache2/conf"
    DIR_APACHE_CONF="/usr/local/apache2/conf"
    HTPASSWD="htpasswd"
else if test -d /etc/apache2 ; then
    DIR_APACHE="/etc/apache2"
    DIR_APACHE_CONF="/etc/apache2/conf.d"
    HTPASSWD="htpasswd2"
else if test -d /etc/httpd/conf ; then
    DIR_APACHE="/etc/httpd/conf"
    DIR_APACHE_CONF="/etc/httpd/conf.d"
    HTPASSWD="htpasswd"
fi
fi
fi
fi

if test -e $DIR_APACHE/apache2.conf ; then
    APACHE_CONF="apache2.conf"
else if test -e $DIR_APACHE/commondhttpd.conf ; then
    APACHE_CONF="commondhttpd.conf"
else if test -e $DIR_APACHE/httpd.conf ; then
    APACHE_CONF="httpd.conf"
fi
fi
fi


WEB_USER=`more $DIR_APACHE/$APACHE_CONF | grep ^User | cut -d " " -f 2`

if [ -z "$WEB_USER"  ] ; then
    for fichier in $DIR_APACHE/*
      do
      if [ -f "$fichier" ];	then
	  WEB_USER_TEMP=`more $fichier | grep ^User | cut -d " " -f 2`
	  if [ -n "$WEB_USER_TEMP" ]; then
	      WEB_USER=$WEB_USER_TEMP
	  fi
      fi
    done
fi

##
## FUNCTION 
##

# Check the user and group nagios and create it if necessary

function check_user_nagios()
{
    nagios_user=`grep nagios /etc/passwd > /dev/null; echo $?`
    if [ $nagios_user == 0 ]; then
	echo "user nagios ... yes"
    else
	echo "user nagios ... no"
	useradd -g nagios nagios
	if [ $? == 0 ]; then
	    echo "creating user nagios ... ok"
	else
	    echo "error creating user nagios check you environnement ..."
	    exit
	fi
    fi
}

function check_group_nagios()
{
    nagios_group=`grep nagios /etc/group > /dev/null; echo $?`
    if [ $nagios_group == 0 ]; then
	echo "group nagios ... yes"
    else
	echo "group nagios... no"
	groupadd nagios
	if [ $? == 0 ]; then
	    echo "creating group nagios ... ok"
	else
	    echo "error creating group nagios check you environnement ..."
	    exit
	fi
    fi
}

function check_group_nagiocmd()
{
    nagios_group=`grep nagiocmd /etc/group > /dev/null; echo $?`
    if [ $nagios_group == 0 ]; then
	echo "group nagiocmd ... yes"
    else
	echo "group nagiocmd ... no"
	groupadd nagiocmd
	if [ $? == 0 ]; then
	    echo "creating group nagiocmd ... ok"
	else
	    echo "error creating group nagiocmd check you environnement ..."
	    exit
	fi
    fi
}


# When exit on error

function error()
{
    echo "ERROR"
    exit 2
}

# Check apache version, and configure it. Ask to restart apache server
# Make a copy of the original file as httpd.conf.initial

function configure_apache()
{
    echo ""
    echo "Configure Apache server ..."
    echo ""

    # configure httpd.conf
    if test -e $DIR_APACHE_CONF/oreon.conf ; then
	echo "apache is already configurated"
    else
	echo "" > $DIR_APACHE_CONF/oreon.conf
	echo "##" >> $DIR_APACHE_CONF/oreon.conf
        echo "## Section add by OREON" >> $DIR_APACHE_CONF/oreon.conf
        echo "##" >> $DIR_APACHE_CONF/oreon.conf
        echo "" >> $DIR_APACHE_CONF/oreon.conf
        echo "Alias /oreon/ $INSTALL_DIR_OREON/" >> $DIR_APACHE_CONF/oreon.conf
        echo "<Directory "/usr/local/oreon">" >> $DIR_APACHE_CONF/oreon.conf
        echo "    Options None" >> $DIR_APACHE_CONF/oreon.conf
        echo "    AllowOverride AuthConfig" >> $DIR_APACHE_CONF/oreon.conf
        echo "    Order allow,deny" >> $DIR_APACHE_CONF/oreon.conf
        echo "    Allow from all" >> $DIR_APACHE_CONF/oreon.conf
        echo "</Directory>" >> $DIR_APACHE_CONF/oreon.conf
        echo "" >> $DIR_APACHE_CONF/oreon.conf
	echo "Apache configurated"
    fi

    # add apache user to nagios group
    usermod -G nagios,nagiocmd,$WEB_USER $WEB_USER >> $LOG_FILE 2>> $LOG_FILE
    echo "user $WEB_USER added to nagios group"
    echo "Apache is now ready for OREON"

    #restart apache !
    if test -x /etc/init.d/apache ; then
	/etc/init.d/apache restart >> $LOG_FILE 2>> $LOG_FILE
    else if test -x /etc/init.d/httpd ; then
	/etc/init.d/httpd restart 
    else if test -e /etc/init.d/apache2 ; then
	/etc/init.d/apache2 restart >> $LOG_FILE 2>> $LOG_FILE
    else
	echo "unable to restart apache server."
    fi
    fi
    fi
    
    echo ""

}

# install OREON interface 

function confirm_oreon()
{
    if test -d $INSTALL_DIR_OREON/ ; then
	echo "Oreon is aldready install on your server !"
	
	echo -n "Are you sure you want to install OREON ? [y/n] "
	read answer
	if [ $answer == 'n' ]; then
	    echo "Ok, so bye bye !! "
	    exit
	else if [ $answer == 'y' ]; then
	    install_oreon
	else
	    echo "Please answer y or n ! "
	    confirm_oreon
	fi
	fi
    else 
    install_oreon
    fi
}

function install_oreon()
{
	echo "install oreon ..."

	if test -d $INSTALL_DIR_OREON ; then
	    echo "Oreon Directory already exists"
	else
	    mkdir $INSTALL_DIR_OREON >> $LOG_FILE 2>> $LOG_FILE
	    echo "Oreon Directory created"
	fi
	cp -Rf $SRC_OREON/* $INSTALL_DIR_OREON  >> $LOG_FILE 2>> $LOG_FILE
	chown -R $WEB_USER:$WEB_USER $INSTALL_DIR_OREON/
	chmod -R 775 $INSTALL_DIR_OREON/* >> $LOG_FILE 2>> $LOG_FILE
	chmod -R 775 $INSTALL_DIR_NAGIOS/  >> $LOG_FILE 2>> $LOG_FILE
	if test -d /etc/nagios ; then
	    chmod -R 775 /etc/nagios >> $LOG_FILE 2>> $LOG_FILE
	fi

# rrdtool directory configuration
 
	chown -R nagios:nagios $INSTALL_DIR_OREON/rrd >> $LOG_FILE 2>> $LOG_FILE
	chmod 775 $INSTALL_DIR_OREON/rrd/*  >> $LOG_FILE 2>> $LOG_FILE

# trafficMap data directory confiiguration

	chown -R nagios:nagios $INSTALL_DIR_OREON/include/trafficMap/average >> $LOG_FILE 2>> $LOG_FILE
	chmod 775 $INSTALL_DIR_OREON/include/trafficMap/average  >> $LOG_FILE 2>> $LOG_FILE
	

# install plugins oreon

	if test -d $INSTALL_DIR_NAGIOS/libexec ; then
	    echo "$INSTALL_DIR_NAGIOS/libexec already exists"
	else
	    mkdir $INSTALL_DIR_NAGIOS/libexec >> $LOG_FILE 2>> $LOG_FILE
	    echo "$INSTALL_DIR_NAGIOS/libexec created"
	fi

	cp plugins/*.* $INSTALL_DIR_NAGIOS/libexec
	echo "Plugins Oreon Added on Nagios Plugins Directory"
	echo ""
	chown -R $WEB_USER:$WEB_USER $INSTALL_DIR_NAGIOS/libexec >> $LOG_FILE 2>> $LOG_FILE
	chmod 775 $INSTALL_DIR_NAGIOS/libexec/* >> $LOG_FILE 2>> $LOG_FILE
	
	
	if test -d $INSTALL_DIR_NAGIOS/etc ; then
	    echo "$INSTALL_DIR_NAGIOS/etc already exists"
	else 
	    mkdir  $INSTALL_DIR_NAGIOS/etc  >> $LOG_FILE 2>> $LOG_FILE
	    echo "$INSTALL_DIR_NAGIOS/etc created"
	fi
	echo ""

	chown -R $WEB_USER:$WEB_USER $INSTALL_DIR_NAGIOS/etc >> $LOG_FILE 2>> $LOG_FILE
	chmod 775 $INSTALL_DIR_NAGIOS/etc >> $LOG_FILE 2>> $LOG_FILE
	
	cp plugins/oreon.conf $INSTALL_DIR_NAGIOS/libexec >> $LOG_FILE 2>> $LOG_FILE
	chmod 775 $INSTALL_DIR_NAGIOS/libexec/oreon.conf >> $LOG_FILE 2>> $LOG_FILE
	chown -R $WEB_USER:$WEB_USER $INSTALL_DIR_NAGIOS/libexec/oreon.conf >> $LOG_FILE 2>> $LOG_FILE

	if test -d $INSTALL_DIR_OREON/nagios_cfg/upload ; then
	    echo "$INSTALL_DIR_OREON/nagios_cfg/upload already exists"
	else 
	    echo "$INSTALL_DIR_OREON/nagios_cfg/upload created"
	    mkdir $INSTALL_DIR_OREON/nagios_cfg/upload
	fi
	chmod 775 $INSTALL_DIR_OREON/nagios_cfg/upload >> $LOG_FILE 2>> $LOG_FILE
	chown -R $WEB_USER:$WEB_USER $INSTALL_DIR_OREON/nagios_cfg/upload >> $LOG_FILE 2>> $LOG_FILE

	if test -d $INSTALL_DIR_OREON/nagios_cfg ; then
	    echo "$INSTALL_DIR_OREON/nagios_cfg already exists"
	else
	    echo "$INSTALL_DIR_OREON/nagios_cfg created"
	    mkdir $INSTALL_DIR_OREON/nagios_cfg
	fi
	chmod 775 $INSTALL_DIR_OREON/nagios_cfg/ >> $LOG_FILE 2>> $LOG_FILE
	chown -R $WEB_USER:$WEB_USER $INSTALL_DIR_OREON/nagios_cfg/ >> $LOG_FILE 2>> $LOG_FILE

	
      	# modify sudoers file
	sudo=`cat $SUDO_FILE | grep OREON > /dev/null; echo $?`
	
	if [ $sudo == '1' ]; then
	    echo "#Add by OREON" >> $SUDO_FILE
	    echo "User_Alias      OREON= $WEB_USER" >> $SUDO_FILE
	    echo "OREON   ALL = NOPASSWD: /etc/init.d/nagios restart" >> $SUDO_FILE
	    echo "" >> $SUDO_FILE
	else
	    echo "sudo is already configurated"
	fi
	
	# restart mysql to be sure that mysqld is running !
	echo "Restart Mysql server ..."
	if test -x /etc/init.d/mysqld ; then
	    /etc/init.d/mysqld restart 
	else if test -x /etc/init.d/mysql ; then
	    /etc/init.d/mysql restart
	else
	    echo "We don't find Mysql server. OREON will not run."
	    exit
	fi
        fi
}

# installation script

check_group_nagios
check_user_nagios
check_group_nagiocmd
configure_apache
confirm_oreon

echo ""
echo ""
echo "##################################################"
echo "# Go to the URL : http://your-server/oreon/      #"
echo "# to finish the setup                            #"
echo "#                                                #"
echo "#        Report bugs at bugs@oreon.org           #"
echo "#                                                #"
echo "#           Thanks for using OREON.              #"
echo "#           -----------------------              #"
echo "#        Contact : infos@oreon.org               #"
echo "#          http://www.oreon.org                  #"
echo "##################################################"
